% RadarMet Rain response simulator - v06
% Formulas from Sauvageot, 1992, p.18,19,146
% F.S. Marzano, SapienzaUniRome, Oct. 2002 - Jan. 2016

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% INPUT
close all; clear all; format compact;

display('>>> RadarMet rain response simulator - v06')

% Input for system parameters
ButtonName=questdlg('Operation frequency band?','Band','C','X','none');
switch ButtonName,
case 'C',
    iband=1;
case 'X',
    iband=2;
end
   
prompt={'Enter transmitter peak power (0.01-1000) [kW]',...
        'Enter transmitted pulse length (0.1-2.0) [micros]',...
        'Enter Pulse repetition frequency (300-4000) [Hz]',...
        'Enter no. pulses to be averaged by receiver (1-128)'...
        'Enter max elevation angles [deg]'};

def={'10','1','1000','32','10'};
dlgTitle='Input for system parameters';
answer1=inputdlg(prompt,dlgTitle,1,def);
input1=char(answer1);
Pt_kW=eval(input1(1,:));
tau=eval(input1(2,:));
prf_Hz=eval(input1(3,:));
n_pulse=eval(input1(4,:));
elev=eval(input1(5,:));

ButtonName=questdlg('Receiver matched filter?','Matched filter','Y','N','none');
switch ButtonName,
case 'Y'
    imatched=1;
case 'N'
    imatched=0
end

if (imatched==0)
    prompt={'6-dB IF bandwidth (1-10) ? [MHz]'};
    def={'2'};
    dlgTitle='Input for bandwidth'
    answer2=inputdlg(prompt,dlgTitle,1,def);
    input2=char(answer2);
    B6dB_MHz=eval(input2(1,:));
end
    
% Input for rainfall parameters

ButtonName=questdlg('Rain rate profile?','Rain Rate','Cell rain','Gaussian','none');
switch ButtonName,
case 'Cell rain',
    Rprofile=1;
case 'Gaussian',
    Rprofile=2;
end

if (Rprofile==1) 
    prompt={'Enter rain rate within the volume resolution [mm/h]'};
    def={'50'};
    dlgTitle='Input for rainfall parameters';
    answer3=inputdlg(prompt,dlgTitle,1,def);
    input3=char(answer3);
    R=eval(input3(1,:));
end

if (Rprofile==2)   
    prompt={'Enter max rain rate [mm/h]',...
        'Enter rain rate profile Gaussian randomization value (0=> NoRand,...,1=> StdRR=1mm/h)'};
    def={'50','0.5'};
    dlgTitle='Input for rainfall parameters';
    answer4=inputdlg(prompt,dlgTitle,1,def);
    input4=char(answer4);
    Rmax=eval(input4(1,:));
    Rrand=eval(input4(2,:));   
end 


% Radar transmitter parameter
if (iband==1) f_GHz=5.6; end
if (iband==2) f_GHz=9.6; end

lambda_cm=30/f_GHz;
dr=(3*10^8)*(tau*10^-6)/2;   %% dr[m]=(c[m/s]*tau[s])/2
Pt=Pt_kW*1000;

% Parametri d'antenna
theta_3dB=1;           % deg
phi_3dB=1;             % deg    
eta_r=1;               % efficienza di radiazione
D0=((4*pi)*(180^2))/((theta_3dB*phi_3dB)*(pi^2));    % direttivit
G0=eta_r*D0;           % guadagno

% Parametri ricevitore
k=1.38*(10^-23);       % Costante di Boltzmann [J/K] 
T0=290;                % [K] 
F_dB=3;                % Figura di rumore  3dB
F_ad=10^(F_dB/10);
Ta = (F_ad-1)*T0;      % Temperatura d'antenna
Tsys=Ta+T0;            % [K]

% Matched filter
match_param=1.2;
if (imatched==1) B6dB_MHz=match_param/tau; end

% Finite-bandwidth power loss(Doviak and Zrnic, 1993, p. 80)
% Lfilter_dB=-2.3;
af=pi/(2*sqrt(log(2))); 
Lfilter_ad=(coth(af*B6dB_MHz*tau)-1/(af*B6dB_MHz*tau))^-1
Lfilter_dB=-10*log10(Lfilter_ad)
B=B6dB_MHz*(10^6);   % Banda del ricevitore [Hz]


% Transmitter chain loss (0-6)[dB]
Ltx_dB=2;
Ltx_ad=10^(-Ltx_dB/10);

% Receiver chain loss (0-6)[dB]
Lrx_dB=2;
Lrx_ad=10^(-Lrx_dB/10);

% Medium parameters
K2=0.93;

% Range
dr_Km=dr/1000;
r=0:dr_Km:dr;          % [Km] si considera dr letto in Km come rmax

% Maximum range
rmax_km=(3*10^8/1000)/(prf_Hz*2);

% Maximum unanbiguous velocity (without unfolding)
vmax_ms=(lambda_cm*0.01/4)*prf_Hz;

% Volume scan time
t_scan=(n_pulse*prf_Hz)*(360/theta_3dB)*(elev/phi_3dB);    %[us]
t_scan_s=t_scan*(10^-6);

%################################################################################RAINFALL

if (Rprofile==1)  
    if (iband==1)      % Banda C
        a=366; b=1.42; % T=0 C
    end
    if (iband==2)      % Banda X
        a=380; b=1.53; % T=0 C
    end
    
    % PROFILO Z-R (Z=a*R^b)
    Z=a.*(R.^b);       % in mm^6m^-3
    Z_dB=10.*log10(Z);
    
    % PROFILO K-R
    L=1;
end

if (Rprofile==2)
     
    % Costruzione del profilo sintetico di rain rate:
    devstd=(max(r)/2)/3.5;
    R=Rmax.*exp(-((r-max(r)/2).^2)./(2*devstd^2));
    Rfix=R;
    Rn=Rfix+Rrand.*abs(randn(size(Rfix)));
    R=Rn;
    
    if (iband==1)      % Banda C
        a=366; b=1.42;
        c=1.90e-3; d=1.23; 	
    end
    if (iband==2)      % Banda X
        a=380; b=1.53;
        c=1.02e-2; d=1.18; 	% T=0 C, orographic rain  (Delrieu et al., JAM, 2000)
    end
    
    % PROFILO Z-R (Z=a*R^b)
    Z=a.*(R.^b);       % in mm^6m^-3
    Z_dB=10*log10(Z);
          
    for i=1:1:length(r)
        
        % PROFILO K-R
        KdBkm(i)=c*(R(i)^d);	      % in dB/km
        Knpkm(i)=KdBkm(i)/4.343;	  % in 1/km (or Np/km)
        L(i)=exp(-sum(Knpkm)*dr_Km);
    end
end

if (Rprofile==2)

    %Baedi et al. (2000)
    LOG_LWC1=((Z_dB/10)-1.76)/5.17;
    LWC1=10.^(LOG_LWC1);

    %Fox and Illingworth (1997)
    LOG_LWC2=((Z_dB./10)-log10(0.012))/1.16;
    LWC2=10.^(LOG_LWC2);
   
    %Sauvageot and Omar (1987)
    LOG_LWC3=((Z_dB/10)-log10(0.03))/1.31;
    LWC3=10.^(LOG_LWC3);

    %Atlas (1954)
    LOG_LWC4=((Z_dB/10)-log10(0.048))/2;
    LWC4=10.^(LOG_LWC4);

    %Best fit of all data for the CAMEX-3 campaign and the CLARE 98's data for the drizzle clouds
    LOG_LWC5=((Z_dB/10)-2.51)/1.58;
    LWC5=10.^(LOG_LWC5);
end


%################################################################################PROCESSING

N=(k*Tsys*B);      % rumore Watt

% Potenza ricevuta Pr[mW]
cost=((pi^5)*10^-19)/((2^10)*1.08*log(2));
C=(tau*theta_3dB*phi_3dB*(G0^2)*Lfilter_ad*Lrx_ad)/(lambda_cm^2);
Pr=Pt*(Ltx_ad)*cost*C.*((K2.*((L.^2).*Z))./(r.^2)); 
Pr_dBm=10*log10(Pr);

% Rapporto segnale rumore single pulse
SNR=Pr*(10^-3)/N;
SNR_dB=10*log10(SNR);

% Rapporto segnale rumore multi pulse
if (n_pulse==1)
    Fact_npulse=1;
else
    Fact_npulse=n_pulse/2;
end
SNRmp=(Pr*(10^-3)/N)*Fact_npulse;
SNRmp_dB=10*log10(SNRmp);


% Rapporto segnale rumore minimo
SNRmin=1;
SNRmin_dB=10*log10(SNRmin);     

% Minimum detectable signal power (MDSP)
Prmin=N*SNRmin;                 
Prmin_mW=Prmin*1000;
Prmin_dBm=10*log10(Prmin_mW);

% Minimum detectable reflectivity factor (MDRF)
Zmin=(r.^2./(Pt*cost*C*K2.*(L.^2)))*N*SNRmin;
Zmin_dBZ=10*log10(Zmin);

% Minimum detectable rainfall (MDRR) - Marshall Palmer Z=200R^1.6
RminMP_mmh=(Zmin/200).^(1/1.6);

RminDA_mmh=(Zmin/a).^(1/b);  %Delrieu et al., JAM, 2000

%##################################################################################RIS

% Maximum range
range=num2str(rmax_km);
ris1=['Maximun range:' range 'Km'] 

% Maximun unanbiguos velocity
v=num2str(vmax_ms);
ris1=['Maximun unanbiguos velocity:' v 'm/s']

% Volume scan time
t=num2str(t_scan_s);
ris=['Volume scan time:' t 's']

%#################################################################################FIGURES

figure
subplot(2,2,1)
plot(r,Pr_dBm);    
hold on; 
x=0:1:max(r);
plot(x,Prmin_dBm,'r-.')
xlabel('Range [km]');
ylabel('Received average power [dBm]')
legend('Pr','Pr min.riv.',0)
grid;
%----------------------------------------------------------------------------------------
Pt_t=num2str(Pt_kW); PRF=num2str(prf_Hz); N_pulse=num2str(n_pulse);

if (iband==1) 
    if (Rprofile==1) 
        title('- C BAND - Cell Rain Profile');
    end
    if (Rprofile==2) 
        title('- C BAND - Gaussian Rain Profile'); 
    end
end

if (iband==2) 
    if (Rprofile==1) 
        title('- X BAND - Cell Rain Profile');
    end
    if (Rprofile==2) 
        title('- X BAND - Gaussian Rain Profile'); 
    end
end
%------------------------------------------------------------------------------------
   
subplot(2,2,2)
plot(r,SNR_dB); 
hold on;
plot(r,SNRmp_dB,'g-.');
plot(x,SNRmin_dB,'r');
xlabel('Range [km]');
ylabel('Received SNR [dB]')
legend(' SNR s.p.','SNR m.p.', 'SNR min.riv.',0)
    grid;

%-------------------------------------------------------------------------------Title
if (Rprofile==1)
    R_t=num2str(R);
    tit=['R=' R_t ' mm/h  ,   Pt=' Pt_t ' kW  ,  PRF=' PRF 'Hz  ,  Npulse=' N_pulse]; 
else
    R_t=num2str(Rmax);
    tit=['Rmax=' R_t ' mm/h  , ' 'Pt=' Pt_t ' kW  ,  PRF=' PRF 'Hz  ,  Npulse=' N_pulse]; 
end
title(tit);
%------------------------------------------------------------------------------------       
        
subplot(2,2,3)
plot(r,Zmin_dBZ);
xlabel('Range [km]');
ylabel('Minimum detectable Z[dBZ]');
grid;

subplot(2,2,4)
plot(r,RminMP_mmh); hold on;
plot(r,RminDA_mmh,'r--')
xlabel('Range [km]');
ylabel('Minimum detectable R[mm/h]')
legend('Rmin.riv.MP','Rmin.riv.DA',0)
grid;




